"""
Common constants for DTO definitions.
"""

from __future__ import annotations

from typing import Final

# Pagination defaults
DEFAULT_PAGE_LIMIT: Final[int] = 50
MAX_PAGE_LIMIT: Final[int] = 1000
